// АГАТ 9 графика

{DEFINE NotOptPixel}           // Без оптимизаций PutPixel
{DEFINE NotOptLine}            // Без оптимизаций Line (классический Брезенхем)


{$IFNDEF NotOptPixel}
  {$DEFINE PixelOpt3}           // Оптимизация 3 - Ассемблер
{$ENDIF}

{$IFNDEF NotOptLine}
  {$DEFINE LineOpt1}           // Оптимизация 1 - Бразенхем оптимизация целых - замена Integer на SmallInt/Byte где возможно
{$ENDIF}

unit GR;

interface

// uses Math;

const
// Выбор платформы
  grcDrvSetAGAT   = $C700;      // Адрес IO установки режимов АГАТ
  grcDrvSetApple  = $C050;      // Адрес IO установки режимов АppleII

// Графические режимы
  grcCMGR     =  $01;           // Цветной среднего разрешения 128х128х16
  grcCHGR     =  $00;           // Цветной высокого разрешения 256х256х4х4палитры
  grcMHGR     =  $03;           // Монохромный высокого разрешения 256х256х2х4палитры
  grcMDGR     =  $83;           // Монохромный двойной плотности 512х256х2х4палитры
  grcACHGR    =  $00;           // Цветной графический высокого разрешения AppleII 280х192х6

// Текстовые режимы
  grcT32      =  $02;           // Текстовый цветной низкого разрешения 32х32х8х4палитры
  grcT64      =  $82;           // Текстовый монохромный высокого разрешения 64х32х2х4палитры
  grcAT40     =  $01;           // Текстовый монохромный среднего разрешения AppleII 40х24х2х4палитры

// Смешанный режим
  grcACHGRT40 =  $02;           // Смешанный цветной графический высокого разрешения и монохромный текстовый AppleII 280х160х6+40х4х2

// Размер страницы
  grcMediumPage = $10;
  grcLargePage  = $20;
  grcTextPage   = $04;

// Максимальное количество страниц
  grcMaxNumMediumPages  = 15;
  grcMaxNumLargePages   =  7;
  grcMaxNumTextPages    = 31;

// Ширина экрана в байтах (Ширина*БитНаПиксель/8бит)
  grcWinthInBytesCMGR   = 128*4 div 8;
  grcWinthInBytesCHGR   = 256*2 div 8;
  grcWinthInBytesMHGR   = 256*1 div 8;
  grcWinthInBytesMDGR   = 512*1 div 8;

// Адреса из ZeroPage для временных переменных AGAT-9
const
  zpTempByte_1  = $9D;
  zpTempByte_2  = zpTempByte_1 + 1;
  zpTempWord_1  = zpTempByte_1;                // Использован для grvScreenPtr (глобальная переменная)

  zpTempByte_3  = zpTempByte_1 + 2;            // Использован для LineXXGR.LineOpt4 (локальные переменные)
  zpTempByte_4  = zpTempByte_1 + 3;            // Использован для LineXXGR.LineOpt4 (локальные переменные)
  zpTempWord_2  = zpTempByte_3;

  zpTempByte_5  = zpTempByte_1 + 4;
  zpTempByte_6  = zpTempByte_1 + 5;
  zpTempWord_3  = zpTempByte_5;                // Использован для LineXXGR.LineOpt4 (локальные переменные)

  zpTempByte_7  = zpTempByte_1 + 6;
  zpTempByte_8  = zpTempByte_1 + 7;
  zpTempWord_4  = zpTempByte_7;                // Использован для LineXXGR.LineOpt4 (локальные переменные)

  zpTempByte_9  = zpTempByte_1 + 8;
  zpTempByte_10 = zpTempByte_1 + 9;
  zpTempWord_5  = zpTempByte_9;                // Использован для LineXXGR.LineOpt4 (локальные переменные)

  zpTempByte_11 = zpTempByte_1 + 10;
  zpTempByte_12 = zpTempByte_1 + 11;
  zpTempWord_6  = zpTempByte_11;

  zpTempByte_13 = zpTempByte_1 + 12;
  zpTempByte_14 = zpTempByte_1 + 13;
  zpTempWord_7  = zpTempByte_13;

var
  grvScrSeg:    Byte absolute $D0;           // Старший байт страницы памяти текущего графического режима
  grvColor:     Byte absolute $30;           // Текущий цвет графического режима
  grvX0:        Byte absolute $E0;           // Текущая координата X0
  grvY0:        Byte absolute $E1;           // Текущая координата Y0
  grvScreenPtr: Word absolute zpTempWord_1;   // Указатель на текущий байт в экранной странице (ADDR.Y)

var
  grvMode    : Byte;               // Текущий Графический режим
  grvPageSize: Byte;               // Размер страницы текущего режима в параграфах (256 байт)
  grvMaxColor: Byte;               // Максимальное количество цветов
  grvMaxX:     Word;               // Максимальное значение координаты X
  grvMaxY:     Word;               // Маскимальное значение координаты Y


  procedure InitGraph(Drv:Word; Mode, NumPage:Byte);  // Drv: платформа, Mode: графический режим, NumPage: номер страницы (отстчёт от 0, берётся по модулю grсМaxNumPages)
  procedure ClearScreen(NumPage: Byte);               // Очистка заданой графической страницы текущим цветом
  procedure SetColor(Color: Byte);                    // Установка текущего цвета

{$IFNDEF PixelOpt3}
  procedure PutPixelCHGR(X,Y: Byte);                  // Установка пикселя в режиме CHGR
  procedure PutPixelMHGR(X,Y: Byte);                  // Установка пикселя в режиме MHGR
{$ELSE}
  procedure PutPixelCHGR(X,Y: Byte); assembler;       // Установка пикселя в режиме CHGR
  procedure PutPixelMHGR(X,Y: Byte); assembler;       // Установка пикселя в режиме MHGR
{$ENDIF}
  procedure LineCHGR(X0,Y0,X1,Y1: Byte);              // Рисование линии в режиме CHGR
  procedure LineMHGR(X0,Y0,X1,Y1: Byte);              // Рисование линии в режиме MHGR


implementation

var
  Tab1BitOnPix  : array [0..7] of Byte = ($80, $40, $20, $10, $08, $04, $02, $01); // Таблица пикселей с цветом 1бит/приксель
  Tab2BitOnPix  : array [0..3] of Byte = ($C0, $30, $0C, $03);                     // Таблица пикселей с цветом 2бит/приксель

  Tab1BitsStart : array [0..7] of Byte = ($FF, $7F, $3F, $1F, $0F, $07, $03, $01); // Таблица стартовых пикселей с цветом 1бит/приксель
  Tab1BitsEnd   : array [0..7] of Byte = ($80, $C0, $E0, $F0, $F8, $FC, $FE, $FF); // Таблица конечных  пикселей с цветом 1бит/приксель
  Tab2BitsStart : array [0..3] of Byte = ($FF, $3F, $0F, $03);                     // Таблица пикселей с цветом 2бит/приксель
  Tab2BitsEnd   : array [0..3] of Byte = ($C0, $F0, $FC, $FF);                     // Таблица пикселей с цветом 2бит/приксель

  grvScreenSegment: Word;       // Адрес текущего сегмента


procedure InitGraph(Drv:Word; Mode, NumPage:Byte);
var
  IOWord: Word;
  NPage: Byte;
begin
  IOWord:= 0;
  grvMaxColor:= 1;
  grvMode:= Mode;
  if (grvMode = grcCMGR) or (grvMode = grcMHGR) then
  begin
      grvPageSize:= grcMediumPage;
      NPage:= NumPage and grcMaxNumMediumPages;
      if NPage > 7 then
      begin
        IOWord:= 8;
        NPage:= NPage and 7;
      end;
      if grvMode = grcCMGR then
        grvMaxColor:= 15;
    end
  else if (grvMode and $03) = 2 then
    begin
      grvPageSize:= grcTextPage;
      NPage:= NumPage and grcMaxNumTextPages;
      if grvMode = grcT32 then
        grvMaxColor:= 7;
    end
  else if (grvMode = grcCHGR) or (grvMode = grcMDGR) then
    begin
      grvPageSize:= grcLargePage;
      NPage:= NumPage and grcMaxNumLargePages;
      if NPage > 3 then
      begin
        IOWord:= 8;
        NPage:= NPage and 3;
      end;
      if grvMode = grcCHGR then
        grvMaxColor:= 3;
    end
  else
    begin
      Exit;
  end;
  IOWord:= Drv + IOWord + Mode + NPage*grvPageSize;
  grvPageSize:= 2*grvPageSize;
  grvScrSeg:= NPage*grvPageSize; // Зависит от маппинга страниц памяти
  grvScreenSegment:= grvScrSeg*256;
  Peek(IOWord);
end;

procedure ClearScreen(NumPage: Byte);
var
  ScreenPtr: Pointer;
  Size: Word;
begin
  Size:= grvPageSize*256;
  ScreenPtr:= Pointer(NumPage*Size);
  FillChar(ScreenPtr, Size, Char(grvColor));
end;

procedure SetColor(Color: Byte);
begin
  grvColor:= Color;
end;

{$IFDEF NotOptPixel}

//-------------------------------------------------------------------------------
procedure PutPixelCHGR(X,Y: Byte);
var
  ScreenPtr: ^Byte;
  Pixel, PixByte: Byte;
begin
  grvX0:= X;
  grvY0:= Y;
  if (grvY0 and $01) <> 0 then
  begin
     ScreenPtr:= Pointer(Word((grvScrSeg + $20)*256 + (grvY0 shr 1)*grcWinthInBytesCHGR + grvX0 div 4));
     Pixel:= $C0 shr (2*(grvX0 mod 4));
     PixByte:= ScreenPtr^;
     Pixel:= ((PixByte xor grvColor) and Pixel) xor PixByte;
     ScreenPtr^:= Pixel;
    end
  else
    begin
     ScreenPtr:= Pointer(Word(grvScrSeg*256 + (grvY0 shr 1)*grcWinthInBytesCHGR + grvX0 div 4));
     Pixel:= $C0 shr (2*(grvX0 mod 4));
     PixByte:= ScreenPtr^;
     Pixel:= ((PixByte xor grvColor) and Pixel) xor PixByte;
     ScreenPtr^:= Pixel;
  end;
end;

//-------------------------------------------------------------------------------
procedure PutPixelMHGR(X,Y: Byte);
var
  ScreenPtr: ^Byte;
  Pixel, PixByte: Byte;
begin
  grvX0:= X;
  grvY0:= Y;
  ScreenPtr:= Pointer(Word(grvScrSeg*256 + grvY0*grcWinthInBytesMHGR + grvX0 div 8));
  Pixel:= $80 shr (grvX0 mod 8);
  PixByte:= ScreenPtr^;
  Pixel:= ((PixByte xor grvColor) and Pixel) xor PixByte;
  ScreenPtr^:= Pixel;
end;


{$ELSE}
  {$IFDEF PixelOpt3}

//-------------------------------------------------------------------------------  
procedure PutPixelCHGR(X,Y: Byte); assembler;
asm
{
// grvX0:= X;
  mva  X grvX0
  sta  grvScreenPtr
  
// Pixel:= Tab2BitOnPix[grvX0 and 3];
  and #%00000011
  tax
  
// Tmp:= Word(grvX0 shl 1)
  lda #$00
  asl  grvScreenPtr
  rol  @
  sta  grvScreenPtr+1
  
// grvY0:= Y;
  mva  Y grvY0
  tay
  
// Tmp:= Tmp + (grvY0 and $FE)*256;
  and #%11111110
  ora  grvScreenPtr+1
  
// ScreenPtr:= Pointer(grvScreenSegment + (Tmp shr 3));
  lsr  @
  ror  grvScreenPtr
  lsr  @
  ror  grvScreenPtr
  lsr  @
  ror  grvScreenPtr
  clc
  adc  grvScrSeg
  sta  grvScreenPtr+1
  
// if Boolean(grvY0 and $01) then
  tya
  and #$01
  beq  EndIf1
  
//  ScreenPtr:= Pointer(Word(ScreenPtr) + $2000);
  lda  grvScreenPtr+1
  add #$20
  sta  grvScreenPtr+1
  
EndIf1:
// PixByte:= ScreenPtr^;
  ldy #$00
  lda  (grvScreenPtr),y
  
// Pixel:= ((PixByte xor grvColor) and Pixel) xor PixByte;
  eor  grvColor
  and  Adr.Tab2BitOnPix,x
  eor  (grvScreenPtr),y
  
// ScreenPtr^:= Pixel;
  sta  (grvScreenPtr),y
};
end;

//-------------------------------------------------------------------------------
procedure PutPixelMHGR(X,Y: Byte); assembler;
asm
{
// grvX0:= X;
  mva  X grvX0
  sta  grvScreenPtr
  
// Pixel:= Tab1BitOnPix[grvX0 and 7];
  and #%00000111
  tax
  
// grvY0:= Y;
  mva  Y grvY0
  
// Tmp:= grvY0*256 + grvX0;
// ScreenPtr:= Pointer(grvScreenSegment + (Tmp shr 3));
  lsr  @
  ror  grvScreenPtr
  lsr  @
  ror  grvScreenPtr
  lsr  @
  ror  grvScreenPtr
  clc
  adc  grvScrSeg
  sta  grvScreenPtr+1
  
// PixByte:= ScreenPtr^;
  ldy #$00
  lda  (grvScreenPtr),y
  
// Pixel:= ((PixByte xor grvColor) and Pixel) xor PixByte;
  eor  grvColor
  and  Adr.Tab1BitOnPix,x
  eor  (grvScreenPtr),y
  
// ScreenPtr^:= Pixel;
  sta  (grvScreenPtr),y
};
end;
  {$ENDIF}
{$ENDIF}

{$IFDEF NotOptLine}

//-------------------------------------------------------------------------------
procedure LineCHGR(X0,Y0,X1,Y1: Byte);

  Procedure Octant0(X0,Y0,DeltaX,DeltaY: Byte; XDirection: SmallInt);
  var
    DeltaYx2: Integer;
    DeltaYx2MinusDeltaXx2: Integer;
    ErrorTerm: Integer;

  begin
    DeltaYx2:= DeltaY * 2;
    DeltaYx2MinusDeltaXx2:= DeltaYx2 - Integer(DeltaX * 2);
    ErrorTerm:= DeltaYx2 - Integer(DeltaX);
    PutPixelCHGR(X0, Y0);
    for DeltaX:= DeltaX downto 1 do
    begin
      if ErrorTerm >= 0 then
      begin
         Inc(Y0);
         Inc(ErrorTerm, DeltaYx2MinusDeltaXx2);
       end
      else
       begin
         Inc(ErrorTerm, DeltaYx2);
      end;
      Inc(X0, XDirection);
      PutPixelCHGR(X0, Y0);
    end;
  end;

  Procedure Octant1(X0,Y0,DeltaX,DeltaY: Byte; XDirection: SmallInt);
  var
    DeltaXx2: Integer;
    DeltaXx2MinusDeltaYx2: Integer;
    ErrorTerm: Integer;

  begin
    DeltaXx2:= DeltaX * 2;
    DeltaXx2MinusDeltaYx2:= DeltaXx2 - Integer(DeltaY * 2);
    ErrorTerm:= DeltaXx2 - Integer(DeltaY);
    PutPixelCHGR(X0, Y0);
    for DeltaY:= DeltaY downto 1 do
    begin
      if ErrorTerm >= 0 then
      begin
         Inc(X0, XDirection);
         Inc(ErrorTerm, DeltaXx2MinusDeltaYx2);
       end
      else
       begin
         Inc(ErrorTerm, DeltaXx2);
      end;
      Inc(Y0);
      PutPixelCHGR(X0, Y0);
    end;
  end;

var
  DeltaX,DeltaY: Integer;
  Temp: Integer;

begin
  if Y0 > Y1 then
  begin
    Temp:= Y0;
    Y0:= Y1;
    Y1:= Temp;
    Temp:= X0;
    X0:= X1;
    X1:= Temp;
  end;
  DeltaX:= X1 - X0;
  DeltaY:= Y1 - Y0;
  if DeltaX > 0 then
  begin
     if DeltaX > DeltaY then
       Octant0(X0,Y0,DeltaX,DeltaY,1)
     else
       Octant1(X0,Y0,DeltaX,DeltaY,1);
   end
  else
   begin
     DeltaX:= -DeltaX;
     if DeltaX > DeltaY then
       Octant0(X0,Y0,DeltaX,DeltaY,-1)
     else
       Octant1(X0,Y0,DeltaX,DeltaY,-1);
  end;
end;


//-------------------------------------------------------------------------------
procedure LineMHGR(X0,Y0,X1,Y1: Byte);

  Procedure Octant0(X0,Y0,DeltaX,DeltaY: Byte; XDirection: SmallInt);
  var
    DeltaYx2: Integer;
    DeltaYx2MinusDeltaXx2: Integer;
    ErrorTerm: Integer;

  begin
    DeltaYx2:= DeltaY * 2;
    DeltaYx2MinusDeltaXx2:= DeltaYx2 - Integer(DeltaX * 2);
    ErrorTerm:= DeltaYx2 - Integer(DeltaX);
    PutPixelMHGR(X0, Y0);
    for DeltaX:= DeltaX downto 1 do
    begin
      if ErrorTerm >= 0 then
      begin
         Inc(Y0);
         Inc(ErrorTerm, DeltaYx2MinusDeltaXx2);
       end
      else
       begin
         Inc(ErrorTerm, DeltaYx2);
      end;
      Inc(X0, XDirection);
      PutPixelMHGR(X0, Y0);
    end;
  end;

  Procedure Octant1(X0,Y0,DeltaX,DeltaY: Byte; XDirection: SmallInt);
  var
    DeltaXx2: Integer;
    DeltaXx2MinusDeltaYx2: Integer;
    ErrorTerm: Integer;

  begin
    DeltaXx2:= DeltaX * 2;
    DeltaXx2MinusDeltaYx2:= DeltaXx2 - Integer(DeltaY * 2);
    ErrorTerm:= DeltaXx2 - Integer(DeltaY);
    PutPixelMHGR(X0, Y0);
    for DeltaY:= DeltaY downto 1 do
    begin
      if ErrorTerm >= 0 then
      begin
         Inc(X0, XDirection);
         Inc(ErrorTerm, DeltaXx2MinusDeltaYx2);
       end
      else
       begin
         Inc(ErrorTerm, DeltaXx2);
      end;
      Inc(Y0);
      PutPixelMHGR(X0, Y0);
    end;
  end;

var
  DeltaX,DeltaY: Integer;
  Temp: Integer;

begin
  if Y0 > Y1 then
  begin
    Temp:= Y0;
    Y0:= Y1;
    Y1:= Temp;
    Temp:= X0;
    X0:= X1;
    X1:= Temp;
  end;
  DeltaX:= X1 - X0;
  DeltaY:= Y1 - Y0;
  if DeltaX > 0 then
  begin
     if DeltaX > DeltaY then
       Octant0(X0,Y0,DeltaX,DeltaY,1)
     else
       Octant1(X0,Y0,DeltaX,DeltaY,1);
   end
  else
   begin
     DeltaX:= -DeltaX;
     if DeltaX > DeltaY then
       Octant0(X0,Y0,DeltaX,DeltaY,-1)
     else
       Octant1(X0,Y0,DeltaX,DeltaY,-1);
  end;
end;

{$ELSE}
  {$IFDEF LineOpt1}

//-------------------------------------------------------------------------------  
procedure LineCHGR(X0,Y0,X1,Y1: Byte);

  Procedure Octant0(X0,Y0,DeltaX,DeltaY: Byte; XDirection: ShortInt);
  var
    DeltaYx2: SmallInt;
    DeltaYx2MinusDeltaXx2: SmallInt;
    ErrorTerm: SmallInt;

  begin
    DeltaYx2:= DeltaY * 2;
    DeltaYx2MinusDeltaXx2:= DeltaYx2 - SmallInt(DeltaX * 2);
    ErrorTerm:= DeltaYx2 - SmallInt(DeltaX);
    PutPixelCHGR(X0, Y0);
    for DeltaX:= DeltaX downto 0 do
    begin
      if ErrorTerm >= 0 then
      begin
         Inc(Y0);
         Inc(ErrorTerm, DeltaYx2MinusDeltaXx2);
       end
      else
       begin
         Inc(ErrorTerm, DeltaYx2);
      end;
      Inc(X0, XDirection);
      PutPixelCHGR(X0, Y0);
    end;
  end;

  Procedure Octant1(X0,Y0,DeltaX,DeltaY: Byte; XDirection: ShortInt);
  var
    DeltaXx2: SmallInt;
    DeltaXx2MinusDeltaYx2: SmallInt;
    ErrorTerm: SmallInt;

  begin
    DeltaXx2:= DeltaX * 2;
    DeltaXx2MinusDeltaYx2:= DeltaXx2 - SmallInt(DeltaY * 2);
    ErrorTerm:= DeltaXx2 - SmallInt(DeltaY);
    PutPixelCHGR(X0, Y0);
    for DeltaY:= DeltaY downto 0 do
    begin
      if ErrorTerm >= 0 then
      begin
         Inc(X0, XDirection);
         Inc(ErrorTerm, DeltaXx2MinusDeltaYx2);
       end
      else
       begin
         Inc(ErrorTerm, DeltaXx2);
      end;
      Inc(Y0);
      PutPixelCHGR(X0, Y0);
    end;
  end;

var
  DeltaX,DeltaY: SmallInt;
  Temp: SmallInt;

begin
  if Y0 > Y1 then
  begin
    Temp:= Y0;
    Y0:= Y1;
    Y1:= Temp;
    Temp:= X0;
    X0:= X1;
    X1:= Temp;
  end;
  DeltaX:= X1 - X0;
  DeltaY:= Y1 - Y0;
  if DeltaX > 0 then
  begin
     if DeltaX > DeltaY then
       Octant0(X0,Y0,DeltaX,DeltaY,1)
     else
       Octant1(X0,Y0,DeltaX,DeltaY,1);
   end
  else
   begin
     DeltaX:= -DeltaX;
     if DeltaX > DeltaY then
       Octant0(X0,Y0,DeltaX,DeltaY,-1)
     else
       Octant1(X0,Y0,DeltaX,DeltaY,-1);
  end;
end;


//-------------------------------------------------------------------------------
procedure LineMHGR(X0,Y0,X1,Y1: Byte);

  Procedure Octant0(X0,Y0,DeltaX,DeltaY: Byte; XDirection: ShortInt);
  var
    DeltaYx2: SmallInt;
    DeltaYx2MinusDeltaXx2: SmallInt;
    ErrorTerm: SmallInt;

  begin
    DeltaYx2:= DeltaY * 2;
    DeltaYx2MinusDeltaXx2:= DeltaYx2 - SmallInt(DeltaX * 2);
    ErrorTerm:= DeltaYx2 - SmallInt(DeltaX);
    PutPixelMHGR(X0, Y0);
    for DeltaX:= DeltaX downto 0 do
    begin
      if ErrorTerm >= 0 then
      begin
         Inc(Y0);
         Inc(ErrorTerm, DeltaYx2MinusDeltaXx2);
       end
      else
       begin
         Inc(ErrorTerm, DeltaYx2);
      end;
      Inc(X0, XDirection);
      PutPixelMHGR(X0, Y0);
    end;
  end;

  Procedure Octant1(X0,Y0,DeltaX,DeltaY: Byte; XDirection: ShortInt);
  var
    DeltaXx2: SmallInt;
    DeltaXx2MinusDeltaYx2: SmallInt;
    ErrorTerm: SmallInt;

  begin
    DeltaXx2:= DeltaX * 2;
    DeltaXx2MinusDeltaYx2:= DeltaXx2 - SmallInt(DeltaY * 2);
    ErrorTerm:= DeltaXx2 - SmallInt(DeltaY);
    PutPixelMHGR(X0, Y0);
    for DeltaY:= DeltaY downto 0 do
    begin
      if ErrorTerm >= 0 then
      begin
         Inc(X0, XDirection);
         Inc(ErrorTerm, DeltaXx2MinusDeltaYx2);
       end
      else
       begin
         Inc(ErrorTerm, DeltaXx2);
      end;
      Inc(Y0);
      PutPixelMHGR(X0, Y0);
    end;
  end;

var
  DeltaX,DeltaY: SmallInt;
  Temp: SmallInt;

begin
  if Y0 > Y1 then
  begin
    Temp:= Y0;
    Y0:= Y1;
    Y1:= Temp;
    Temp:= X0;
    X0:= X1;
    X1:= Temp;
  end;
  DeltaX:= X1 - X0;
  DeltaY:= Y1 - Y0;
  if DeltaX > 0 then
  begin
     if DeltaX > DeltaY then
       Octant0(X0,Y0,DeltaX,DeltaY,1)
     else
       Octant1(X0,Y0,DeltaX,DeltaY,1);
   end
  else
   begin
     DeltaX:= -DeltaX;
     if DeltaX > DeltaY then
       Octant0(X0,Y0,DeltaX,DeltaY,-1)
     else
       Octant1(X0,Y0,DeltaX,DeltaY,-1);
  end;
end;

  {$ENDIF}
{$ENDIF}

end.

